﻿open GameState
open System
open System.Drawing
open System.Security.Permissions
open System.Windows.Forms

let fps = 30
let spf = 1.f / (float32 fps)
let mspf = 1000 / fps

type DrawPanel =
    inherit Panel
    new () as this = {} then this.DoubleBuffered <- true

// The assembly attributes
[<assembly: RegistryPermission(SecurityAction.RequestMinimum, All = "HKEY_CURRENT_USER")>]
do ()

// The main function
[<STAThread>]
do
    let form = new Form()
    form.Width <- 640
    form.Height <- 480
    form.FormBorderStyle <- FormBorderStyle.Fixed3D
    form.MinimizeBox <- false
    form.MaximizeBox <- false
    form.Text <- "Dare Islands - LD48h #17 - http://www.xmunkki.org/"
    
    let draw_panel = new DrawPanel()
    draw_panel.Dock <- DockStyle.Fill
    form.Controls.Add draw_panel
    
    let timer = new Timer()
    timer.Interval <- mspf

    // Create the game state
    let cur_state = ref (GameStateGame.new_game draw_panel.ClientSize.Width draw_panel.ClientSize.Height)
    
    let mouse_state = ref empty_mouse_state
    let mouse_x = ref 0
    let mouse_y = ref 0
    let mouse_left = ref false
    let mouse_right = ref false
    
    // Define the draw function
    let do_draw (g:Graphics) =
        (!cur_state).gs_draw g
        
    // Define the update function
    let do_update () =
        // Update the mouse state
        mouse_state := { 
            !mouse_state with
                mouse_x = float32 !mouse_x;
                mouse_y = float32 !mouse_y;
                mouse_left_down = !mouse_left;
                mouse_left_pressed = !mouse_left && not ((!mouse_state).mouse_left_down);
                mouse_left_released = not !mouse_left && (!mouse_state).mouse_left_down;
                mouse_right_down = !mouse_right;
                mouse_right_pressed = !mouse_right && not ((!mouse_state).mouse_right_down);
                mouse_right_released = not !mouse_right && (!mouse_state).mouse_right_down }
        
        // Update the game state
        match (!cur_state).gs_update spf !mouse_state with
        | GSU_Normal -> ()
        | GSU_Change s -> cur_state := s
        | GSU_Quit -> form.Close()
        
        // Redraw
        draw_panel.Invalidate()
    
    // Add the event handles and start the update loop
    let update_mouse_pos (args:MouseEventArgs) =
        mouse_x := args.X
        mouse_y := args.Y
    
    draw_panel.Paint.Add(fun a -> do_draw a.Graphics)
    draw_panel.MouseMove.Add(fun a -> update_mouse_pos a)
    draw_panel.MouseDown.Add(fun a -> 
        update_mouse_pos a
        if a.Button = MouseButtons.Left then mouse_left := true
        if a.Button = MouseButtons.Right then mouse_right := true)
    draw_panel.MouseUp.Add(fun a ->
        update_mouse_pos a
        if a.Button = MouseButtons.Left then mouse_left := false
        if a.Button = MouseButtons.Right then mouse_right := false)
    timer.Tick.Add (fun _ -> do_update())
    timer.Enabled <- true
    
    Application.Run(form)